/*=============================================================================
 * Tansitive closure program. Query results with given first argument (bf).
 *
 * This version uses the SLDMagic transformation and tables all predicates.
 * It uses load_dync to load the data.
 *
 * The main program is in 'bench_lib.P'.
 * This file is a modified version of a file distributed with OpenRuleBench.
 *=============================================================================
 */

/*-----------------------------------------------------------------------------
 * ID of this Benchmark:
 *-----------------------------------------------------------------------------
 */

bench_id('tcbf_sldmdc').


/*-----------------------------------------------------------------------------
 * Options:
 *-----------------------------------------------------------------------------
 */

/* Data load method: */
:- import load_dync/2 from consult.
load_data(Filename) :- load_dync(Filename,a).

/* Index for the given database relation (edges of the graph): */
:- index(par/2, trie).

/* Tabling method: */
:- use_subsumptive_tabling p0/1.
:- use_subsumptive_tabling p1/1.
:- use_subsumptive_tabling tc/2.


/*-----------------------------------------------------------------------------
 * Benchmark Program:
 *-----------------------------------------------------------------------------
 */

/*
  tc(X,Y) :- par(X,Y).
  tc(X,Y) :- par(X,Z), tc(Z,Y).
*/

/* Result of the SLDMagic Transformation: */
p1(A) :- p1(B), par(B,A).
p1(A) :- par(1,A).
p0(A) :- p1(B), par(B,A).
p0(A) :- par(1,A).
tc(1,A) :- p0(A).


/*-----------------------------------------------------------------------------
 * Benchmark Query:
 *-----------------------------------------------------------------------------
 */

bench :- tc(1,_), fail.
bench.


/*-----------------------------------------------------------------------------
 * Benchmark Query with Writing of Result:
 *-----------------------------------------------------------------------------
 */

bench_write(Stream) :-
	tc(1,X),
	write(Stream, X),
	write(Stream, '\n'),
	fail.
bench_write(_).


/*-----------------------------------------------------------------------------
 * The rest is the same for all XSB benchmarks:
 *-----------------------------------------------------------------------------
 */

:- consult('bench_lib.P').

