/*=============================================================================
 * Tansitive closure program. Query results with given first argument (bf).
 *
 * Benchmark IDs: tcbf_tab_*
 * This version tables the predicate tc to ensure termination for cyclic data.
 *
 * Test results will be recorded in file 'RUN_BENCH__RESULT'
 * This file is a modified version of a file distributed with OpenRuleBench.
 *=============================================================================
 */

/*-----------------------------------------------------------------------------
 * Options:
 *-----------------------------------------------------------------------------
 */

:- table(tc/2).

/*-----------------------------------------------------------------------------
 * Benchmark Program:
 *-----------------------------------------------------------------------------
 */

tc(X,Y) :- par(X,Y).
tc(X,Y) :- par(X,Z), tc(Z,Y).

/*-----------------------------------------------------------------------------
 * Benchmark Query:
 *-----------------------------------------------------------------------------
 */

bench :- tc(1,_), fail.
bench.

/*-----------------------------------------------------------------------------
 * List of Test Data Files:
 *-----------------------------------------------------------------------------
 */

test_file(tcbf_tab_f1, '../data/tc_d1000_parsize50000_xsb_nocyc.P').
test_file(tcbf_tab_f2, '../data/tc_d1000_parsize50000_xsb_cyc.P').
test_file(tcbf_tab_f3, '../data/tc_d1000_parsize250000_xsb_nocyc.P').
test_file(tcbf_tab_f4, '../data/tc_d1000_parsize250000_xsb_cyc.P').
test_file(tcbf_tab_f5, '../data/tc_d1000_parsize500000_xsb_nocyc.P').
test_file(tcbf_tab_f6, '../data/tc_d1000_parsize500000_xsb_cyc.P').
test_file(tcbf_tab_f7, '../data/tc_d2000_parsize500000_xsb_nocyc.P').
test_file(tcbf_tab_f8, '../data/tc_d2000_parsize500000_xsb_cyc.P').
test_file(tcbf_tab_f9, '../data/tc_d2000_parsize1000000_xsb_nocyc.P').
test_file(tcbf_tab_f10, '../data/tc_d2000_parsize1000000_xsb_cyc.P').


/*=============================================================================
 * The rest is the same for all YAP benchmarks:
 *=============================================================================
 */


/*-----------------------------------------------------------------------------
 * Main Predicate: Run the Test and Write File RUN__BENCH_RESULT.
 *-----------------------------------------------------------------------------
 */

test(BenchID) :- 
	test_file(BenchID, Filename),
	open('RUN_BENCH__RESULT', write, Output),
	% Write Benchmark ID to output file:
	write(Output, BenchID),
	write(Output, '\t'),
	% Get times before Load (Real in ms, CPU in s):
	statistics(walltime, [Real0,_]),
	CPU0 is cputime,
	% Load Data:
	[Filename],
	% Get times after Load:
	CPU1 is cputime,
	statistics(walltime, [Real1,_]),
	% Compute Load times:
	LoadCPU is floor((CPU1 - CPU0) * 1000),
	LoadReal is Real1 - Real0,
	% Write load CPU time to output file:
	write(Output, LoadCPU),
	write(Output, '\t'),
	flush_output(Output),
	% Get times before Exec (Real in ms, CPU in s):
	statistics(walltime, [Real3,_]),
	CPU3 is cputime,
	% NOW THE BENCHMARK QUERY:
	bench,
	% Get times after Exec:
	CPU4 is cputime,
	statistics(walltime, [Real4,_]),
	% Compute Times for Benchmark Execution:
	ExecCPU is floor((CPU4 - CPU3) * 1000),
	ExecReal is Real4 - Real3,
	% Write Exec CPU time to output file:
	write(Output, ExecCPU),
	write(Output, '\t'),
	% Write total CPU time to output file:
	TotalCPU is floor(CPU4 * 1000),
	write(Output, TotalCPU),
	write(Output, '\t'),
	% Compare CPU times with real times, write "Warn" status if strange:
	status(LoadCPU, LoadReal, ExecCPU, ExecReal, Status),
	write(Output, Status),
	write(Output, '\t'),
	close(Output).

/*-----------------------------------------------------------------------------
 * Comparison of Wallclock Time and CPU Time:
 *-----------------------------------------------------------------------------
 */

/* If the two times differ to much, we output status 'Warn': */

status(LoadCPU, LoadReal, _ExecCPU, _ExecReal, 'Warn1') :-
	LoadCPU > LoadReal,
	!.
status(LoadCPU, LoadReal, _ExecCPU, _ExecReal, 'Warn2') :-
	LoadReal > LoadCPU * 1.15,
	!.
status(_LoadCPU, _LoadReal, ExecCPU, ExecReal, 'Warn3') :-
	ExecCPU > ExecReal,
	!.
status(_LoadCPU, _LoadReal, ExecCPU, ExecReal, 'Warn4') :-
	ExecReal > ExecCPU * 1.15,
	!.
status(_LoadCPU, _LoadReal, _ExecCPU, _ExecReal, 'OK').


/*-----------------------------------------------------------------------------
 * Main Goal: Access Command Line Argument and Call Test Predicate:
 *-----------------------------------------------------------------------------
 */

:- unix(argv(ArgumentList)),
	ArgumentList = [BenchID | _],
	test(BenchID).

